/*
 * @Copyright(C): 2016-2021 ROS小课堂 www.corvin.cn
 * @Author: adam_zhuo
 * @Date: 2021-04-30 15:29:58
 * @Description: imu小程序
 * @History: 20210429:初始化文件-adam_zhuo
 */

#include "imu_demo.h"

/**
 * @description: 构造函数初始化变量。
 */
Imu_Demo::Imu_Demo(){
    
    start_flag = false;
    _angle_sub = nh.subscribe<std_msgs::Float32>("/angle", 1, &Imu_Demo::angle_callback, this);
    _yaw_sub = nh.subscribe<std_msgs::Float32>("/yaw_data", 1, &Imu_Demo::yaw_callback, this);
    _cmd_vel_pub = nh.advertise<geometry_msgs::Twist>("/cmd_vel", 1);
}

Imu_Demo::~Imu_Demo(){

}

/**
 * @description: 接收指定角度回调函数。程序启动后,发送指定角度，任意放置小车，小车最终朝向都会是指定角度。
 */
void Imu_Demo::angle_callback(const std_msgs::Float32::ConstPtr &data){
    start_flag = true;
    target_radian = data->data/180*M_PI;
    target_radian_copy = target_radian;
    if(target_radian < 0)
        target_radian += (M_PI * 2);
}

/**
 * @description: yaw角回调函数，不断检测当前小车朝向与指定角度的差值，如果大于一个阈值，则控制小车旋转。
 */
void Imu_Demo::yaw_callback(const std_msgs::Float32::ConstPtr &data){

    //如果没有接收到发送的指定角度，则不做任何处理直接return
    if(!start_flag)
        return;
    
    //获取当前yaw角值
    current_radian = data->data ;
    current_radian_copy = current_radian;

    // >>>判断与指定角度的差值，控制小车旋转
    

    
    // <<<判断与指定角度的差值，控制小车旋转

    //向小车发送控制命令
    _cmd_vel_pub.publish(twist);
}

int main(int argc, char **argv){

    //初始化节点
    ros::init(argc, argv, "imu_demo_node");
    //根据Imu_Demo声明对象，调用构造函数
    Imu_Demo imu_demo;
    //使ROS中回调函数生效
    ros::spin();
}