#include <ros/ros.h>
#include <actionlib/client/simple_action_client.h>
#include "ros_test_pkg/myActionAction.h"


void finishCB(const actionlib::SimpleClientGoalState &state,
        const ros_test_pkg::myActionResultConstPtr &result)
{
    ROS_WARN("Action status: [%s], result: [%d]", state.toString().c_str(), result->result_num);
    ros::shutdown();
}

void startCB()
{
    ROS_INFO("ActionServer start action...");
}

void feedbackCB(const ros_test_pkg::myActionFeedbackConstPtr &feedback)
{
    ROS_INFO("Client get feedback: [%d]", feedback->current_num);
}

int main(int argc, char **argv)
{
    ros::init(argc, argv, "action_client");
    if(argc != 3)
    {
        ROS_INFO("%d", argc);
        ROS_WARN("Usage: action_client <goal> <time>");
        return 1;
    }

    actionlib::SimpleActionClient<ros_test_pkg::myActionAction> ac("action_server", true);
    ROS_INFO("Waiting for action server to start...");
    ac.waitForServer();

    ros_test_pkg::myActionGoal goal;
    goal.goal_num = atoi(argv[1]);
    ROS_INFO("Sending Goal [%d] and Preempt time of [%d]s", goal.goal_num, atoi(argv[2]));
    ac.sendGoal(goal, &finishCB, &startCB, &feedbackCB);

    bool finished_before_timeout =ac.waitForResult(ros::Duration(atoi(argv[2])));
    actionlib::SimpleClientGoalState state = ac.getState();
    ROS_INFO("Action status: [%s]", state.toString().c_str());
    if(!finished_before_timeout)
    {
        ac.cancelGoal();
        ROS_INFO("Action status:[%s]", state.toString().c_str());
        ROS_ERROR("Action did't finish before the timeout.");
    }

    return 0;
}

