#include <ros/ros.h>
#include <actionlib/server/simple_action_server.h>
#include "ros_test_pkg/myActionAction.h"

class myActionAction
{
    protected:
        ros::NodeHandle nh_;
        actionlib::SimpleActionServer<ros_test_pkg::myActionAction> as;

        ros_test_pkg::myActionFeedback feedback;
        ros_test_pkg::myActionResult result;

        std::string action_name;
        int goal;
        int progress;

    public:
        myActionAction(std::string name):
            as(nh_, name, boost::bind(&myActionAction::executeCB, this, _1), false),
            action_name(name)
    {
        as.registerPreemptCallback(boost::bind(&myActionAction::preemptCB, this));
        as.start();
    }

    void preemptCB()
    {
        ROS_ERROR("[%s] get preempted !", action_name.c_str());
        result.result_num = progress;
        as.setPreempted(result, "I got Preempted");
    }

    void executeCB(const ros_test_pkg::myActionGoalConstPtr &goal)
    {
        ROS_INFO("%s is processing the goal %d", action_name.c_str(), goal->goal_num);

        ros::Rate rate(5);
        for(progress = 1; progress <= goal->goal_num; progress++)
        {
            if(!as.isActive() || as.isPreemptRequested()) return;

            if(goal->goal_num == progress)
            {
                result.result_num = progress;
                as.setSucceeded(result);
                ROS_INFO("[%s] Succeeded Goal [%d]", action_name.c_str(), progress);
            }
            else
            {
                feedback.current_num = progress;
                as.publishFeedback(feedback);
                ROS_INFO("Server send feedback: [%d]", feedback.current_num);
            }
            rate.sleep();
        }
    }
};

int main(int argc, char** argv)
{
    ros::init(argc, argv, "action_server");

    myActionAction action_obj(ros::this_node::getName());
    ROS_INFO("Starting Action Server ...");

    ros::spin();
    return 0;
}

